*==============================================================================
* Procedure:		XTabRep
* Purpose:			Print a horizontally paginated cross-tab report
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	01/01/97
* Parameters:		tcColumn - the heading to print for the first column
*					tcHeader - the header to print on each page
*					tcOutput - the output device (if it isn't passed, 'PREVIEW'
*						is used
*					tnMargin - the left margin for the report (if it isn't
*						passed, 0 is used)
* Environment in:	the selected table has 3 columns (the names of which are
*						unimportant) -- the first column contains the "row"
*						values, the second column contains the "column" values,
*						and the third column contains the count. The table can
*						have additional columns, but they're ignored.
* Environment out:	the report has been printed
*==============================================================================

parameters tcColumn, ;
	tcHeader, ;
	tcOutput, ;
	tnMargin
private lcOutput, ;
	lnMargin, ;
	lcField1, ;
	lcField2, ;
	lcField3, ;
	lcAlias, ;
	lnTotColumns, ;
	lnCurrSelect, ;
	lcTotals, ;
	lcXTab, ;
	lcIndex, ;
	lcKey, ;
	lcReport, ;
	lnWidth1, ;
	lnWidth2, ;
	lnCurrAscii

* These variables need to be defined as PRIVATE even in VFP because some
* subroutines use these values.

private pcHeader, ;
	pcColumn, ;
	paColumns, ;
	pnColsPerPage, ;
	pnTotalHPages, ;
	pcVertKey, ;
	plDoneVert, ;
	pnHPage

* Define some constants.

#define cnMAX_COL_WIDTH         20
#define ccTOTALS                'Totals'

* If the output device isn't specified, use "PREVIEW". Otherwise, use the
* appropriate output clause for REPORT FORM.

do case
	case type('tcOutput') <> 'C' or empty(tcOutput) or ;
		upper(tcOutput) = 'PREVIEW'
		lcOutput = 'PREVIEW'
	case upper(tcOutput) = 'PRINT'
		lcOutput = 'TO PRINT NOCONSOLE'
	otherwise
		lcOutput = 'TO FILE ' + tcOutput + ;
			iif('Visual' $ version(), ' ASCII', '') + ' NOCONSOLE'
endcase

* If the report header and first column heading aren't specified, use blank
* values instead. Use 0 for the left margin if it isn't specified.

pcHeader = iif(type('tcHeader') = 'C', tcHeader, '')
pcColumn = iif(type('tcColumn') = 'C', tcColumn, '')
lnMargin = iif(type('tnMargin') = 'N', tnMargin, 0)

* Create an array of headings to print in horizontal columns (ignore blank
* entries since they might be there because of records in the first column
* that didn't have any record for the second column in the source tables).
* Determine the total number of columns we'll be printing (it'll be the number
* of unique columns plus one for the "totals" column).

lcField1 = field(1)
lcField2 = field(2)
lcField3 = field(3)
lcAlias  = alias()
select &lcField2 ;
	from (lcAlias) ;
	where not empty(&lcField2) ;
	into array paColumns ;
	group by 1
lnTotColumns = _tally + 1
dimension paColumns[lnTotColumns]
paColumns[lnTotColumns] = ccTOTALS

* Create a cursor from the cross-tab table because we'll be adding records
* (ignore records with a blank first field since they might be there because
* of records in the second column that didn't have any record for the first
* column in the source tables).

lnCurrSelect = select()
select 0
lcCursor = CreateUniqueAlias()
lcXTab   = CreateUniqueAlias()
select &lcField1, ;
		&lcField2, ;
		&lcField3, ;
		'1' as SORT ;
	from (lcAlias) ;
	where not empty(&lcField1) ;
	into cursor (lcCursor)
use (dbf(lcCursor)) again alias (lcXTab) in 0
use in (lcCursor)

* Create totals records for the rows.

lcTotals = CreateUniqueAlias()
select &lcField1, ;
		ccTOTALS as &lcField2, ;
		sum(&lcField3) as &lcField3, ;
		'1' as SORT ;
	from (lcAlias) ;
	where not empty(&lcField1) ;
	into cursor (lcTotals) ;
	group by 1, 2
select (lcXTab)
append from (dbf(lcTotals))

* Add a blank row that'll appear just above the totals row.

insert into (lcXTab) (SORT) values ('8')

* Create totals records for the columns.

select ccTOTALS as &lcField1, ;
		&lcField2, ;
		sum(&lcField3) as &lcField3, ;
		'9' as SORT ;
	from (lcXTab) ;
	into cursor (lcTotals) ;
	group by 1, 2
select (lcXTab)
append from (dbf(lcTotals))
use in (lcTotals)

* Index the table on the first two columns.

lcIndex = sys(3) + '.IDX'
lcKey   = 'SORT+' + lcField1 + '+' + lcField2
index on &lcKey to (lcIndex) compact

* Create an FRX to use for the report. Define the number of horizontal columns
* per page and the total number of horizontal pages needed.

lcReport = sys(3)
lnWidth1 = len(evaluate(lcField1))
calculate max(len(trim(&lcField2))) to lnWidth2
pnColsPerPage = CreateReport(lcReport, lnWidth1, ;
	min(lnWidth2, cnMAX_COL_WIDTH), lcOutput = 'TO FILE', ;
	iif(lnWidth2 > cnMAX_COL_WIDTH, 2, 1), lnMargin)
pnTotalHPages = ceiling(lnTotColumns/pnColsPerPage)

* Define some variables so they exist during the report. pcVertKey contains
* the key for the first record on this horizontal page, plDoneVert is a flag
* to indicate when we're done vertical pages, and pnHPage is the current
* horizontal page.

pcVertKey  = ''
plDoneVert = .F.
pnHPage    = 0

* Run the report.

#IF 'Visual' $ version()
lnCurrAscii = _asciicols
_asciicols  = 160
#ENDIF
report form (lcReport) &lcOutput

* Clean up and exit.

#IF 'Visual' $ version()
_asciicols = lnCurrAscii
#ENDIF
use
select (lnCurrSelect)
erase (lcIndex)
erase (lcReport + '.FRX')
erase (lcReport + '.FRT')

*==============================================================================
* Function:			GetColumn
* Purpose:			UDF called from the column headings in the FRX: gets the
*						heading for the specified column from the paColumns
*						array
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	01/01/97
* Parameters:		tnColumn - the column number
* Returns:			the column heading for the specified column or a blank
*						value if there is no such column
* Environment in:	pnHPage and pnColsPerPage have been defined
*					the paColumns array contains the column headings
* Environment out:	none
*==============================================================================

function GetColumn
parameters tnColumn
private lnColumn
lnColumn = pnHPage * pnColsPerPage + tnColumn
return iif(lnColumn > alen(paColumns), '', paColumns[lnColumn])

*==============================================================================
* Function:			GetCell
* Purpose:			UDF called from each cell in the FRX: gets the value for
*						the specified column from the current row
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	01/01/97
* Parameters:		tnColumn - the column number
* Returns:			the column heading for the specified column or a blank
*						value if there is no such column
* Environment in:	pnHPage, pnColsPerPage, and plDoneVert have been defined
*					the paColumns array contains the column headings
*					the cross-tab cursor is selected and indexed
* Environment out:	the cross-tab cursor may have been positioned to a new
*						record
*==============================================================================

function GetCell
parameters tnColumn
private lnColumn, ;
	luReturn, ;
	lnRecno, ;
	lcXTab1, ;
	lcXTab2
lnColumn = pnHPage * pnColsPerPage + tnColumn

* If there is no such column or we're done the vertical pages, return a blank
* value.

if lnColumn > alen(paColumns) or plDoneVert
	luReturn = ''

* Save the current record pointer and get the values for the current row and
* column.

else
	lnRecno = recno()
	lcXTab1 = evaluate(field(1))
	lcXTab2 = paColumns[lnColumn]
	do case

* Sort "8" means the blank row just before the totals row, so return a blank
* value.

		case SORT = '8'
			luReturn = ''

* Find the appropriate record in the cursor (either a totals or a regular
* record) and get the value of its "cell" field.

		case seek(iif(trim(lcXTab1) == ccTOTALS, '9', '1') + lcXTab1 + lcXTab2)
			luReturn = evaluate(field(3))

* We couldn't find an appropriate record, so use a 0 and return to the former
* record.

		otherwise
			luReturn = 0
			go lnRecno
	endcase
endif lnColumn > alen(paColumns) ...
return luReturn

*==============================================================================
* Function:			NewPage
* Purpose:			UDF called from the FRX: handle a new page
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	01/01/97
* Parameters:		none
* Returns:			a blank value
* Environment in:	pnHPage, pnTotalHPages, and pcVertKey have been defined
*					the cross-tab cursor is selected and indexed
* Environment out:	pnHPage is set to the next horizontal page
*					plDoneVert is set to .F.
*					if this is the first horizontal page, pcVertKey is set to
*						the key value for the current record
*					if this isn't the first horizontal page, the cross-tab
*						cursor is positioned to the same record as the first
*						row for the first horizontal page for this vertical
*						page
*==============================================================================

function NewPage
pnHPage    = ((_pageno - 1) % pnTotalHPages)
plDoneVert = .F.
if pnHPage = 0
	pcVertKey = evaluate(key(val(sys(21))))
else
	seek pcVertKey
endif pnHPage = 0
return ''

*==============================================================================
* Function:			NewRow
* Purpose:			UDF called from the FRX: handle the end of a row
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	01/01/97
* Parameters:		none
* Returns:			a blank value
* Environment in:	pnHPage, pnTotalHPages, plDoneVert, and pcVertKey have been
*						defined
*					the cross-tab cursor is selected and indexed
* Environment out:	the cross-tab cursor is positioned to the record before the
*						next row for the current horizontal and vertical page
*						(it's the record before because the FRX does its own
*						SKIP)
*					if there are no more records but we're not done all the
*						horizontal pages, plDoneVert is set to .T. (we're done
*						the vertical pages) and the cross-tab cursor is
*						positioned to the same record as the first row for the
*						first horizontal page for this vertical page
*==============================================================================

function NewRow
private lcField, ;
	lcValue
lcField = field(1)
lcValue = evaluate(lcField)
scan while &lcField = lcValue
endscan while &lcField = lcValue
if eof() and pnHPage + 1 < pnTotalHPages
	plDoneVert = .T.
	seek pcVertKey
else
	skip -1
endif eof() ...
return ''

*==============================================================================
* Function:			CreateReport
* Purpose:			Create a report (FRX) file for horizontally paginated
*						reports
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	05/08/97
* Parameters:		tcReport     - the name of the report file to create
*					tnWidth1     - the width for the first column
*					tnWidth2     - the width for the second and subsequent
*						columns
*					tlText       - .T. to output in the width for a text file
*					tnHeadHeight - the height for column headings
*					tnMargin     - the left margin to use
* Returns:			the number of columns per page
* Environment in:	a table (which one is unimportant) is open in the current
*						workarea
* Environment out:	the specified report has been created
*==============================================================================

function CreateReport
parameters tcReport, ;
	tnWidth1, ;
	tnWidth2, ;
	tlText, ;
	tnHeadHeight, ;
	tnMargin
private lnCurrSelect, ;
	lcReport, ;
	lnRepWidth, ;
	lnHeader, ;
	lnDetail, ;
	lnHeaderBand, ;
	lnDetailBand, ;
	lnHPos, ;
	lnColumn, ;
	lcPicture, ;
	lnWidth

* These variables need to be defined as PRIVATE even in VFP because some
* subroutines use these values.

private pnVPosFactor, ;
	pnHFieldFactor, ;
	pnHTextFactor, ;
	pnHPosFactor, ;
	pnWFieldFactor, ;
	pnWTextFactor

* Create the report file.

lnCurrSelect = select()
create report (tcReport) from (alias())
select 0
lcReport = CreateUniqueAlias()
use (tcReport + '.FRX') alias (lcReport) exclusive
zap

* Define the report font and size and horizontal calculation factor. cnFACTOR
* is the number of report units per pixel. It's calculated as report units (a
* value used for all measurements inside the FRX) per inch (10,000) divided by
* points per inch (12) divided by pixels per point (8).

#define ccFONT        'Courier New'
#define cnSIZE        10
#define cnFACTOR      10000.0000/(12 * 8)	&& 104.1666

* Calculate some factors for positioning and size. Although some of the
* factors give the same results, they are stored in different variables in
* case things change in the future.

pnVPosFactor   = (fontmetric(1, ccFONT, cnSIZE) + ;
	fontmetric(5, ccFONT, cnSIZE) + 2) * cnFACTOR
pnHFieldFactor = (fontmetric(1, ccFONT, cnSIZE) + ;
	fontmetric(5, ccFONT, cnSIZE) + 2) * cnFACTOR
pnHTextFactor  = (fontmetric(1, ccFONT, cnSIZE) + ;
	fontmetric(5, ccFONT, cnSIZE)) * cnFACTOR
pnHPosFactor   = fontmetric(7, ccFONT, cnSIZE) * cnFACTOR
pnWFieldFactor = fontmetric(7, ccFONT, cnSIZE) * cnFACTOR
pnWTextFactor  = fontmetric(6, ccFONT, cnSIZE) * cnFACTOR

* The width of the report in characters is the width of the report in 0.1 mm
* (as returned by either PRTINFO(3) or PRTINFO(4), depending on whether we're
* using portrait or landscape, which is returned in PRTINFO(1)) divided by 254
* (0.1 mm per inch) to give inches, less 0.5" (2 x 0.25" left and right
* margins for a typical laser printer), and multiplied by the number of
* characters per inch (10000/pnHPosFactor).

do case

* Portrait letter or legal.

	case inlist(prtinfo(2), 1, 5) and prtinfo(1) = 0
		lnWidth = 8.5

* Landscape letter.

	case prtinfo(2) = 1
		lnWidth = 11

* Landscape legal.

	case prtinfo(2) = 5
		lnWidth = 14

* Non-predefined sizes: landscape or portrait.

	case prtinfo(1) = 1
		lnWidth = prtinfo(3)/254
	otherwise
		lnWidth = prtinfo(4)/254
endcase
lnRepWidth = (lnWidth - 0.5) * 10000/pnHPosFactor

* Find out where the each band starts. Column headers will go on line 5 and the
* detail band will be 3 rows plus the header height lower than that.

lnHeader     = 5
lnDetail     = lnHeader + tnHeadHeight + 3
lnHeaderBand = 1
lnDetailBand = 2

* Put in the report header and band records.

do InsertHeader with lcReport
do InsertBand   with lcReport, 1, 'Page Header', lnDetail - 1
do InsertBand   with lcReport, 2, 'Detail',      1
do InsertBand   with lcReport, 3, 'Page Footer', 0

* Put in a record for the NewPage() object.

do InsertField with lcReport, lnHeaderBand, 'NewPage()', 1, 1, 0, tnMargin

* Insert records for the header band text and fields.

do InsertField with lcReport, lnHeaderBand, ;
	"iif(type('pcHeader') = 'C', trim(pcHeader), '')", 0, 1, 0, tnMargin, ;
	'Center', .T., 'C', '"@I"'
do InsertField with lcReport, lnHeaderBand, 'date()', ;
	iif(set('CENTURY') = 'ON', 10, 8), 1, 2, tnMargin
do InsertText  with lcReport, lnHeaderBand, '"Page"', 4, 2, 0
do InsertField with lcReport, lnHeaderBand, '_pageno', 3, 1, 2, 0, 'Right', ;
	.F., .F., 'J'
do InsertLine  with lcReport, lnHeaderBand, 0, lnHeader - 1, tnMargin
do InsertLine  with lcReport, lnHeaderBand, 0, lnHeader + tnHeadHeight, ;
	tnMargin

* Add a line just above the details with a Print When so it only appears for
* the "totals" record. Use "lnDetail - 0.5" instead of "lnDetail - 1" for a
* report with less space between rows.

do InsertLine  with lcReport, lnDetailBand, 0, lnDetail - 1, tnMargin
replace SUPEXPR with "SORT = '9'", NOREPEAT with .T.

* Put the first table column in the first column of the report.

do InsertField with lcReport, lnHeaderBand, 'pcColumn', tnWidth1, ;
	tnHeadHeight, lnHeader, tnMargin
do InsertField with lcReport, lnDetailBand, ;
	'iif(plDoneVert, "", evaluate(field(1)))', tnWidth1, 1, lnDetail, tnMargin
lnHPos = tnMargin + tnWidth1 + 1

* Put in the columns for the report.

lnColumn  = 1
lcPicture = '"' + NumericPicture(tnWidth2) + '"'
do while lnHPos + tnWidth2 < lnRepWidth
	do InsertField with lcReport, lnHeaderBand, 'GetColumn(' + ;
		ltrim(str(lnColumn)) + ')', tnWidth2, tnHeadHeight, lnHeader, lnHPos, ;
		'Right'
	do InsertField with lcReport, lnDetailBand, 'GetCell(' + ;
		ltrim(str(lnColumn)) + ')', tnWidth2, 1, lnDetail, lnHPos, 'Right', ;
		.F., '', lcPicture
	lnHPos   = lnHPos + tnWidth2 + 1
	lnColumn = lnColumn + 1
enddo while lnHPos + tnWidth2 < lnRepWidth

* Put in a record at the end of each row.

do InsertField with lcReport, lnDetailBand, 'NewRow()', 1, 1, lnDetail, ;
	lnHPos, 'Right'

* Update the report width in some of the existing records.

lnWidth = lnHPos - 1 - tnMargin
replace WIDTH with lnWidth * pnWFieldFactor for WIDTH = 0 and ;
	OBJTYPE <> 9 and OBJTYPE <> 21 and OBJTYPE <> 18 and OBJTYPE <> 1
lnWidth = lnHPos - 1
locate for EXPR = '"Page"'
replace HPOS with (lnWidth - 7.6667) * pnHPosFactor
locate for EXPR = '_pageno'
replace HPOS with (lnWidth - 3) * pnHPosFactor

* Put in the trailer records.

do InsertTrailer with lcReport
if 'Visual' $ version()
	do InsertDE with lcReport
endif 'Visual' $ version()
use
select (lnCurrSelect)
return lnColumn - 1

*********************
function InsertHeader
*********************

parameters tcReport
insert into (tcReport) ;
		(PLATFORM, ;
		UNIQUEID, ;
		OBJTYPE, ;
		OBJCODE, ;
		VPOS, ;
		WIDTH, ;
		FONTFACE, ;
		FONTSIZE, ;
		RULER, ;
		GRID, ;
		GRIDV, ;
		GRIDH, ;
		ADDALIAS, ;
		CURPOS) ;
	values ;
		('WINDOWS', ;
		sys(2015), ;
		1, ;
		53, ;
		1, ;
		-1, ;
		ccFONT, ;
		cnSIZE, ;
		1, ;
		.T., ;
		12, ;
		12, ;
		.T., ;
		.T.)
return

**********************
function InsertTrailer
**********************

parameters tcReport
insert into (tcReport) ;
		(PLATFORM, ;
		UNIQUEID, ;
		OBJTYPE, ;
		VPOS, ;
		HPOS, ;
		HEIGHT, ;
		WIDTH, ;
		PENRED, ;
		FONTFACE, ;
		FONTSIZE) ;
	values ;
		('WINDOWS', ;
		sys(2015), ;
		23, ;
		16, ;
		8, ;
		12, ;
		8, ;
		4, ;
		ccFONT, ;
		cnSIZE)
return

*****************
function InsertDE
*****************

parameters tcReport
insert into (tcReport) ;
		(PLATFORM, ;
		UNIQUEID, ;
		OBJTYPE, ;
		NAME, ;
		EXPR) ;
	values ;
		('WINDOWS', ;
		sys(2015), ;
		25, ;
		'dataenvironment', ;
		'Name = "Dataenvironment"')
return

********************
function InsertField
********************

parameters tcReport, ;
	tnBand, ;
	tcExpression, ;
	tnWidth, ;
	tnHeight, ;
	tnVPos, ;
	tnHPos, ;
	tcAlign, ;
	tlStretch, ;
	tcStyle, ;
	tcPicture
private lnAlign, ;
	lcPicture, ;
	lcStyle, ;
	lnWidth, ;
	lnVPos, ;
	lnHPos
do case
	case type('tcAlign') <> 'C' or empty(tcAlign) or tcAlign = 'Left'
		lnAlign = 0
	case tcAlign = 'Right'
		lnAlign = 1
	otherwise
		lnAlign = 2
endcase
lcPicture = iif(empty(tcPicture), '', tcPicture)
lcStyle   = iif(empty(tcStyle),   '', tcStyle)
lnWidth   = tnWidth * pnWFieldFactor
lnVPos    = tnVPos  * pnVPosFactor + (tnBand - 1) * cnFACTOR
lnHPos    = tnHPos  * pnHPosFactor
insert into (tcReport) ;
		(PLATFORM, ;
		UNIQUEID, ;
		OBJTYPE, ;
		OBJCODE, ;
		EXPR, ;
		HEIGHT, ;
		WIDTH, ;
		VPOS, ;
		HPOS, ;
		PICTURE, ;
		STYLE, ;
		PENRED, ;
		PENGREEN, ;
		PENBLUE, ;
		FILLRED, ;
		FILLGREEN, ;
		FILLBLUE, ;
		MODE, ;
		TOP, ;
		RESETTOTAL, ;
		SUPRPCOL, ;
		STRETCH, ;
		OFFSET, ;
		SUPALWAYS, ;
		FONTFACE, ;
		FONTSIZE) ;
	values ;
		('WINDOWS', ;
		sys(2015), ;
		8, ;
		0, ;
		tcExpression, ;
		pnHFieldFactor * tnHeight, ;
		lnWidth, ;
		lnVPos, ;
		lnHPos, ;
		lcPicture, ;
		lcStyle, ;
		-1, ;
		-1, ;
		-1, ;
		-1, ;
		-1, ;
		-1, ;
		1, ;
		.T., ;
		1, ;
		3, ;
		tlStretch, ;
		lnAlign, ;
		.T., ;
		ccFONT, ;
		cnSIZE)
return

*******************
function InsertText
*******************

parameters tcReport, ;
	tnBand, ;
	tcExpression, ;
	tnWidth, ;
	tnVPos, ;
	tnHPos, ;
	tcAlign, ;
	tcStyle
private lnAlign, ;
	lcStyle, ;
	lnWidth, ;
	lnVPos, ;
	lnHPos
do case
	case type('tcAlign') <> 'C' or empty(tcAlign) or tcAlign = 'Left'
		lnAlign = 0
	case tcAlign = 'Right'
		lnAlign = 1
	otherwise
		lnAlign = 2
endcase
lcStyle = iif(empty(tcStyle), '', tcStyle)
lnWidth = tnWidth * pnWTextFactor
lnVPos  = tnVPos  * pnVPosFactor + tnBand * cnFACTOR
lnHPos  = tnHPos  * pnHPosFactor
insert into (tcReport) ;
		(PLATFORM, ;
		UNIQUEID, ;
		OBJTYPE, ;
		OBJCODE, ;
		EXPR, ;
		HEIGHT, ;
		WIDTH, ;
		VPOS, ;
		HPOS, ;
		STYLE, ;
		PENRED, ;
		PENGREEN, ;
		PENBLUE, ;
		FILLRED, ;
		FILLGREEN, ;
		FILLBLUE, ;
		MODE, ;
		TOP, ;
		RESETTOTAL, ;
		SUPRPCOL, ;
		STRETCH, ;
		OFFSET, ;
		SUPALWAYS, ;
		FONTFACE, ;
		FONTSIZE) ;
	values ;
		('WINDOWS', ;
		sys(2015), ;
		5, ;
		0, ;
		tcExpression, ;
		pnHTextFactor, ;
		lnWidth, ;
		lnVPos, ;
		lnHPos, ;
		lcStyle, ;
		-1, ;
		-1, ;
		-1, ;
		-1, ;
		-1, ;
		-1, ;
		1, ;
		.T., ;
		1, ;
		3, ;
		.T., ;
		lnAlign, ;
		.T., ;
		ccFONT, ;
		cnSIZE)
return

*******************
function InsertBand
*******************

parameters tcReport, ;
	tnBand, ;
	tcBand, ;
	tnHeight
private lnCode
do case
	case tcBand = 'Page Header'
		lnCode = 1
	case tcBand = 'Detail'
		lnCode = 4
	case tcBand = 'Page Footer'
		lnCode = 7
	case tcBand = 'Summary'
		lnCode = 8
endcase
insert into (tcReport) ;
		(PLATFORM, ;
		UNIQUEID, ;
		OBJTYPE, ;
		OBJCODE, ;
		HEIGHT) ;
	values ;
		('WINDOWS', ;
		sys(2015), ;
		9, ;
		lnCode, ;
		tnHeight * pnVPosFactor + (tnBand - 1) * cnFACTOR)
return

*******************
function InsertLine
*******************

parameters tcReport, ;
	tnBand, ;
	tnWidth, ;
	tnVPos, ;
	tnHPos
insert into (tcReport) ;
		(PLATFORM, ;
		UNIQUEID, ;
		OBJTYPE, ;
		OBJCODE, ;
		HEIGHT, ;
		WIDTH, ;
		VPOS, ;
		HPOS, ;
		FLOAT, ;
		PENSIZE, ;
		PENPAT, ;
		SUPALWAYS) ;
	values ;
		('WINDOWS', ;
		sys(2015), ;
		6, ;
		0, ;
		cnFACTOR, ;
		tnWidth * pnWFieldFactor, ;
		tnVPos * pnVPosFactor + (tnBand - 1) * cnFACTOR + 8 * cnFACTOR, ;
		tnHPos * pnHPosFactor, ;
		.T., ;
		1, ;
		8, ;
		.T.)
return

*==============================================================================
* Function:			CreateUniqueAlias
* Purpose:			Creates a unique alias
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	01/01/97
* Parameters:		none
* Returns:			a unique alias
* Environment in:	none
* Environment out:	none
*==============================================================================

function CreateUniqueAlias
private lcAlias
lcAlias = sys(2015)
do while used(lcAlias)
	lcAlias = sys(2015)
enddo while used(lcAlias)
return lcAlias

*===========================================================================
* Function:			NumericPicture
* Purpose:			Calculate the picture for a numeric field with commas
* Author:			Doug Hennig
* Copyright:		(c) 1996 Stonefield Systems Group Inc.
* Last Revision:	10/14/96
* Parameters:		tnWidth - the width of the field
* Returns:			a numeric picture string with commas
* Environment in:	none
* Environment out:	none
* Routines used:	none
*===========================================================================

function NumericPicture
parameters tnWidth
private lcPicture
lcPicture = '9'
do while len(lcPicture) < tnWidth
	if mod(len(lcPicture) + 1, 4) = 0
		lcPicture = '9,' + lcPicture
	else
		lcPicture = '9' + lcPicture
	endif mod(len(lcPicture) ...
enddo while len(lcPicture) < tnWidth
return lcPicture
